@@ -21,8 +21,8 @@ any_apply <- function(xs, f) {
 }
 
 deparse_names <- function(x) {
-  x <- map_if(x, is_quosure, quo_expr)
-  x <- map_if(x, is_formula, f_rhs)
+  x <- map_if(x, is_quosure, quo_squash)
+  x <- map_if(x, is_bare_formula, f_rhs)
   map_chr(x, deparse)
 }
 
@@ -98,3 +98,36 @@ compact_null <- function(x) {
 paste_line <- function(...) {
   paste(chr(...), collapse = "\n")
 }
+
+abort_if_not <- function(...) {
+  tryCatch(stopifnot(...), simpleError = function(e) abort(e))
+}
+
+# Until fixed upstream. `vec_data()` should not return lists from data
+# frames.
+dplyr_vec_data <- function(x) {
+  out <- vec_data(x)
+
+  if (is.data.frame(x)) {
+    new_data_frame(out, n = nrow(x))
+  } else {
+    out
+  }
+}
+
+# Until vctrs::new_data_frame() forwards row names automatically
+dplyr_new_data_frame <- function(x = data.frame(),
+                                 n = NULL,
+                                 ...,
+                                 row.names = NULL,
+                                 class = NULL) {
+  row.names <- row.names %||% .row_names_info(x, type = 0L)
+
+  new_data_frame(
+    x,
+    n = n,
+    ...,
+    row.names = row.names,
+    class = class
+  )
+}
